@@ -28,6 +28,7 @@ static char **mount_files(struct error *, const char *, const struct nvc_contain
 static char **mount_driverstore_files(struct error *, const char *, const struct nvc_container *, const char *, const char *[], size_t);
 static char *mount_directory(struct error *, const char *, const struct nvc_container *, const char *);
 static char *mount_firmware(struct error *, const char *, const struct nvc_container *, const char *);
+static char *mount_in_root(struct error *err, const char *src, const char *rootfs, const char *path, uid_t uid, uid_t gid, unsigned long mountflags);
 static char *mount_with_flags(struct error *, const char *, const char *,  uid_t, uid_t, unsigned long);
 static char *mount_device(struct error *, const char *, const struct nvc_container *, const struct nvc_device_node *);
 static char *mount_ipc(struct error *, const char *, const struct nvc_container *, const char *);
@@ -39,7 +40,6 @@ static int  update_app_profile(struct error *, const struct nvc_container *, dev
 static void unmount(const char *);
 static int  symlink_library(struct error *, const char *, const char *, const char *, uid_t, gid_t);
 static int  symlink_libraries(struct error *, const struct nvc_container *, const char * const [], size_t);
-static void filter_libraries(const struct nvc_driver_info *, char * [], size_t *);
 static int  device_mount_dxcore(struct nvc_context *, const struct nvc_container *);
 static int  device_mount_native(struct nvc_context *, const struct nvc_container *, const struct nvc_device *);
 static int  cap_device_mount(struct nvc_context *, const struct nvc_container *, const char *);
@@ -49,12 +49,9 @@ static char *
 mount_directory(struct error *err, const char *root, const struct nvc_container *cnt, const char *dir)
 {
         char src[PATH_MAX];
-        char dst[PATH_MAX];
         if (path_join(err, src, root, dir) < 0)
                 return (NULL);
-        if (path_resolve_full(err, dst, cnt->cfg.rootfs, dir) < 0)
-                return (NULL);
-        return mount_with_flags(err, src, dst, cnt->uid, cnt->gid, MS_NOSUID|MS_NOEXEC);
+        return mount_in_root(err, src, cnt->cfg.rootfs, dir, cnt->uid, cnt->gid, MS_NOSUID|MS_NOEXEC);
 }
 
 // mount_firmware mounts the specified firmware file. The path specified is the container path and is resolved
@@ -63,12 +60,19 @@ static char *
 mount_firmware(struct error *err, const char *root, const struct nvc_container *cnt, const char *container_path)
 {
         char src[PATH_MAX];
-        char dst[PATH_MAX];
         if (path_resolve_full(err, src, root, container_path) < 0)
                 return (NULL);
-        if (path_join(err, dst, cnt->cfg.rootfs, container_path) < 0)
+        return mount_in_root(err, src, cnt->cfg.rootfs, container_path, cnt->uid, cnt->gid, MS_RDONLY|MS_NODEV|MS_NOSUID);
+}
+
+// mount_in_root bind mounts the specified src to the specified location in a root.
+// If the destination resolves outside of the root an error is raised.
+static char *
+mount_in_root(struct error *err, const char *src, const char *rootfs, const char *path, uid_t uid, uid_t gid, unsigned long mountflags) {
+        char dst[PATH_MAX];
+        if (path_resolve_full(err, dst, rootfs, path) < 0)
                 return (NULL);
-        return mount_with_flags(err, src, dst, cnt->uid, cnt->gid, MS_RDONLY|MS_NODEV|MS_NOSUID);
+        return mount_with_flags(err, src, dst, uid, gid, mountflags);
 }
 
 // mount_with_flags bind mounts the specified src to the specified dst with the specified mount flags
@@ -111,6 +115,8 @@ mount_files(struct error *err, const char *root, const struct nvc_container *cnt
                 return (NULL);
         if (file_create(err, dst, NULL, cnt->uid, cnt->gid, MODE_DIR(0755)) < 0)
                 return (NULL);
+        if (path_new(err, dst, dir) < 0)
+                return (NULL);
         src_end = src + strlen(src);
         dst_end = dst + strlen(dst);
 
@@ -124,19 +130,16 @@ mount_files(struct error *err, const char *root, const struct nvc_container *cnt
                         continue;
                 if (path_append(err, src, paths[i]) < 0)
                         goto fail;
+                if (file_mode_nofollow(err, src, &mode) < 0)
+                        goto fail;
+                // If we encounter resolved directories or symlinks here, we raise an error.
+                if (S_ISDIR(mode) || S_ISLNK(mode)) {
+                        error_setx(err, "unexpected source file mode %o for %s", mode, paths[i]);
+                        goto fail;
+                }
                 if (path_append(err, dst, file) < 0)
                         goto fail;
-                if (file_mode(err, src, &mode) < 0)
-                        goto fail;
-                if (file_create(err, dst, NULL, cnt->uid, cnt->gid, mode) < 0)
-                        goto fail;
-
-                log_infof("mounting %s at %s", src, dst);
-                if (xmount(err, src, dst, NULL, MS_BIND, NULL) < 0)
-                        goto fail;
-                if (xmount(err, NULL, dst, NULL, MS_BIND|MS_REMOUNT | MS_RDONLY|MS_NODEV|MS_NOSUID, NULL) < 0)
-                        goto fail;
-                if ((*ptr++ = xstrdup(err, dst)) == NULL)
+                if ((*ptr++ = mount_in_root(err, src, cnt->cfg.rootfs, dst, cnt->uid, cnt->gid, MS_RDONLY|MS_NODEV|MS_NOSUID)) == NULL)
                         goto fail;
                 *src_end = '\0';
                 *dst_end = '\0';
@@ -558,27 +561,6 @@ symlink_libraries(struct error *err, const struct nvc_container *cnt, const char
         return (0);
 }
 
-static void
-filter_libraries(const struct nvc_driver_info *info, char * paths[], size_t *size)
-{
-        char *lib, *maj;
-
-        /*
-         * XXX Filter out any library that matches the major version of RM to prevent us from
-         * running into an unsupported configurations (e.g. CUDA compat on Geforce or non-LTS drivers).
-         */
-        for (size_t i = 0; i < *size; ++i) {
-                lib = basename(paths[i]);
-                if ((maj = strstr(lib, ".so.")) != NULL) {
-                        maj += strlen(".so.");
-                        if (strncmp(info->nvrm_version, maj, strspn(maj, "0123456789")))
-                                continue;
-                }
-                paths[i] = NULL;
-        }
-        array_pack(paths, size);
-}
-
 static int
 device_mount_dxcore(struct nvc_context *ctx, const struct nvc_container *cnt)
 {
@@ -765,20 +747,12 @@ nvc_driver_mount(struct nvc_context *ctx, const struct nvc_container *cnt, const
                 goto fail;
 
         /* Container library mounts */
-        if (cnt->libs != NULL && cnt->nlibs > 0) {
-                size_t nlibs = cnt->nlibs;
-                char **libs = array_copy(&ctx->err, (const char * const *)cnt->libs, cnt->nlibs);
-                if (libs == NULL)
-                        goto fail;
-
-                filter_libraries(info, libs, &nlibs);
-                if ((tmp = (const char **)mount_files(&ctx->err, cnt->cfg.rootfs, cnt, cnt->cfg.libs_dir, libs, nlibs)) == NULL) {
-                        free(libs);
+        if ((cnt->flags & OPT_CUDA_COMPAT_MODE_MOUNT) && cnt->libs != NULL && cnt->nlibs > 0) {
+                if ((tmp = (const char **)mount_files(&ctx->err, cnt->cfg.rootfs, cnt, cnt->cfg.libs_dir, cnt->libs, cnt->nlibs)) == NULL) {
                         goto fail;
                 }
                 ptr = array_append(ptr, tmp, array_size(tmp));
                 free(tmp);
-                free(libs);
         }
 
         /* Firmware mounts */
